@@ -3,7 +3,7 @@
 //! By default, when simply running CKB, CKB will connect to the official public Nervos network.
 //!
 //! In order to run a chain different to the official public one,
-//! with a config file specifying chain = "path" under [ckb].
+//! with a config file specifying `spec = { file = "<the-path-of-spec-file>" }` under `[chain]`.
 //!
 
 // Because the limitation of toml library,
@@ -14,6 +14,7 @@ use crate::consensus::{
     build_genesis_dao_data, build_genesis_epoch_ext, Consensus, ConsensusBuilder,
     SATOSHI_CELL_OCCUPIED_RATIO, SATOSHI_PUBKEY_HASH, TYPE_ID_CODE_HASH,
 };
+use ckb_constant::hardfork::{mainnet, testnet};
 use ckb_crypto::secp::Privkey;
 use ckb_hash::{blake2b_256, new_blake2b};
 use ckb_jsonrpc_types::Script;
@@ -25,8 +26,9 @@ use ckb_resource::{
 use ckb_types::{
     bytes::Bytes,
     core::{
-        capacity_bytes, BlockBuilder, BlockNumber, BlockView, Capacity, Cycle, EpochNumber,
-        EpochNumberWithFraction, Ratio, ScriptHashType, TransactionBuilder, TransactionView,
+        capacity_bytes, hardfork::HardForkSwitch, BlockBuilder, BlockNumber, BlockView, Capacity,
+        Cycle, EpochNumber, EpochNumberWithFraction, Ratio, ScriptHashType, TransactionBuilder,
+        TransactionView,
     },
     h256, packed,
     prelude::*,
@@ -34,15 +36,19 @@ use ckb_types::{
 };
 use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
-use std::convert::TryFrom;
 use std::error::Error;
 use std::fmt;
 use std::sync::Arc;
 
 pub use error::SpecError;
+pub use hardfork::HardForkConfig;
 
 pub mod consensus;
 mod error;
+mod hardfork;
+
+#[cfg(test)]
+mod tests;
 
 // Just a random secp256k1 secret key for dep group input cell's lock
 const SPECIAL_CELL_PRIVKEY: H256 =
@@ -59,6 +65,7 @@ pub const OUTPUT_INDEX_SECP256K1_BLAKE160_MULTISIG_ALL: u64 = 4;
 
 /// The CKB block chain specification
 #[derive(Clone, PartialEq, Eq, Debug, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
 pub struct ChainSpec {
     /// The spec name, also used identify network
     pub name: String,
@@ -164,6 +171,7 @@ pub mod default_params {
 
 /// Parameters for CKB block chain
 #[derive(Default, Clone, PartialEq, Eq, Debug, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
 pub struct Params {
     /// The initial_primary_epoch_reward
     ///
@@ -220,6 +228,11 @@ pub struct Params {
     /// See [`orphan_rate_target`](consensus/struct.Consensus.html#structfield.orphan_rate_target)
     #[serde(skip_serializing_if = "Option::is_none")]
     pub orphan_rate_target: Option<(u32, u32)>,
+    /// The parameters for hard fork features.
+    ///
+    /// See [`hardfork_switch`](consensus/struct.Consensus.html#structfield.hardfork_switch)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub hardfork: Option<HardForkConfig>,
 }
 
 impl Params {
@@ -293,6 +306,7 @@ impl Params {
 /// The genesis information
 /// Load from config file.
 #[derive(Clone, PartialEq, Eq, Debug, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
 pub struct Genesis {
     /// The genesis block version
     pub version: u32,
@@ -338,6 +352,7 @@ pub struct Genesis {
 
 /// The system cell information
 #[derive(Clone, PartialEq, Eq, Debug, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
 pub struct SystemCell {
     // NOTE: must put `create_type_id` before `file` otherwise this struct can not serialize
     /// whether crate type script
@@ -350,6 +365,7 @@ pub struct SystemCell {
 
 /// The genesis cell information
 #[derive(Clone, PartialEq, Eq, Debug, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
 pub struct GenesisCell {
     /// The genesis cell message
     pub message: String,
@@ -359,6 +375,7 @@ pub struct GenesisCell {
 
 /// Initial token supply cell
 #[derive(Clone, PartialEq, Eq, Debug, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
 pub struct IssuedCell {
     /// The cell capacity
     pub capacity: Capacity,
@@ -368,6 +385,7 @@ pub struct IssuedCell {
 
 /// The genesis dep_group file resources
 #[derive(Clone, PartialEq, Eq, Debug, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
 pub struct DepGroupResource {
     /// The dep_group name
     pub name: String,
@@ -377,6 +395,7 @@ pub struct DepGroupResource {
 
 /// The burned 25% of Nervos CKBytes in genesis block
 #[derive(Clone, PartialEq, Eq, Debug, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
 pub struct SatoshiGift {
     /// The lock pubkey hash
     pub satoshi_pubkey_hash: H160,
@@ -431,16 +450,16 @@ impl ChainSpec {
             return Err(SpecLoadError::file_not_found());
         }
         let config_bytes = resource.get()?;
-        let mut spec: ChainSpec = toml::from_slice(&config_bytes)?;
 
+        let mut spec: ChainSpec = toml::from_slice(&config_bytes)?;
         if let Some(parent) = resource.parent() {
             for r in spec.genesis.system_cells.iter_mut() {
                 r.file.absolutize(parent)
             }
         }
-
         // leverage serialize for sanitizing
         spec.hash = packed::Byte32::new(blake2b_256(&toml::to_vec(&spec)?));
+
         Ok(spec)
     }
 
@@ -459,10 +478,25 @@ impl ChainSpec {
         Ok(())
     }
 
+    /// Completes all parameters for hard fork features and creates a hard fork switch.
+    ///
+    /// Verify the parameters for mainnet and testnet, because all start epoch numbers
+    /// for mainnet and testnet are fixed.
+    fn build_hardfork_switch(&self) -> Result<HardForkSwitch, Box<dyn Error>> {
+        let config = self.params.hardfork.as_ref().cloned().unwrap_or_default();
+        match self.name.as_str() {
+            mainnet::CHAIN_SPEC_NAME => config.complete_mainnet(),
+            testnet::CHAIN_SPEC_NAME => config.complete_testnet(),
+            _ => config.complete_with_default(EpochNumber::MAX),
+        }
+        .map_err(Into::into)
+    }
+
     /// Build consensus instance
     ///
     /// [Consensus](consensus/struct.Consensus.html)
     pub fn build_consensus(&self) -> Result<Consensus, Box<dyn Error>> {
+        let hardfork_switch = self.build_hardfork_switch()?;
         let genesis_epoch_ext = build_genesis_epoch_ext(
             self.params.initial_primary_epoch_reward(),
             self.genesis.compact_target,
@@ -492,6 +526,7 @@ impl ChainSpec {
             .permanent_difficulty_in_dummy(self.params.permanent_difficulty_in_dummy())
             .max_block_proposals_limit(self.params.max_block_proposals_limit())
             .orphan_rate_target(self.params.orphan_rate_target())
+            .hardfork_switch(hardfork_switch)
             .build();
 
         Ok(consensus)
@@ -539,7 +574,8 @@ impl ChainSpec {
             .parent_hash(self.genesis.parent_hash.pack())
             .timestamp(self.genesis.timestamp.pack())
             .compact_target(self.genesis.compact_target.pack())
-            .uncles_hash(self.genesis.uncles_hash.pack())
+            .extra_hash(self.genesis.uncles_hash.pack())
+            .epoch(EpochNumberWithFraction::new_unchecked(0, 0, 0).pack())
             .dao(dao)
             .nonce(u128::from_le_bytes(self.genesis.nonce.to_le_bytes()).pack())
             .transaction(cellbase_transaction)
@@ -612,6 +648,15 @@ impl ChainSpec {
                         .into());
                     }
                 }
+                ScriptHashType::Data1 => {
+                    if !data_hashes.contains_key(&lock_script.code_hash()) {
+                        return Err(format!(
+                            "Invalid lock script: code_hash={}, hash_type=data1",
+                            lock_script.code_hash(),
+                        )
+                        .into());
+                    }
+                }
             }
         }
 
@@ -884,7 +929,7 @@ pub fn build_genesis_type_id_script(output_index: u64) -> packed::Script {
 
 pub(crate) fn build_type_id_script(input: &packed::CellInput, output_index: u64) -> packed::Script {
     let mut blake2b = new_blake2b();
-    blake2b.update(&input.as_slice());
+    blake2b.update(input.as_slice());
     blake2b.update(&output_index.to_le_bytes());
     let mut ret = [0; 32];
     blake2b.finalize(&mut ret);
@@ -907,263 +952,3 @@ pub fn calculate_block_reward(epoch_reward: Capacity, epoch_length: BlockNumber)
         }
     })
 }
-
-#[cfg(test)]
-pub mod test {
-    use super::*;
-    use serde::{Deserialize, Serialize};
-    use std::collections::HashMap;
-
-    #[derive(Clone, Debug, Serialize, Deserialize)]
-    struct SystemCell {
-        pub path: String,
-        pub index: usize,
-        pub data_hash: H256,
-        pub type_hash: Option<H256>,
-    }
-
-    #[derive(Clone, Debug, Serialize, Deserialize)]
-    struct DepGroups {
-        pub included_cells: Vec<String>,
-        pub tx_hash: H256,
-        pub index: usize,
-    }
-
-    #[derive(Clone, Debug, Serialize, Deserialize)]
-    struct SpecHashes {
-        pub genesis: H256,
-        pub cellbase: H256,
-        pub system_cells: Vec<SystemCell>,
-        pub dep_groups: Vec<DepGroups>,
-    }
-
-    fn load_spec_by_name(name: &str) -> ChainSpec {
-        let res = if name == "ckb" {
-            Resource::bundled("specs/mainnet.toml".to_string())
-        } else {
-            let base_name = &name[4..];
-            Resource::bundled(format!("specs/{}.toml", base_name))
-        };
-
-        ChainSpec::load_from(&res).expect("load spec by name")
-    }
-
-    #[test]
-    fn test_bundled_specs() {
-        let bundled_spec_err: &str = r#"
-            Unmatched Bundled Spec.
-
-            Forget to generate docs/hashes.toml? Try to run;
-
-                ckb list-hashes -b > docs/hashes.toml
-        "#;
-
-        let spec_hashes: HashMap<String, SpecHashes> =
-            toml::from_str(include_str!("../../docs/hashes.toml")).unwrap();
-
-        for (name, spec_hashes) in spec_hashes.iter() {
-            let spec = load_spec_by_name(name);
-            assert_eq!(name, &spec.name, "{}", bundled_spec_err);
-            if let Some(genesis_hash) = &spec.genesis.hash {
-                assert_eq!(genesis_hash, &spec_hashes.genesis, "{}", bundled_spec_err);
-            }
-
-            let consensus = spec.build_consensus();
-            if let Err(err) = consensus {
-                panic!("{}", err);
-            }
-            let consensus = consensus.unwrap();
-            let block = consensus.genesis_block();
-            let cellbase = block.transaction(0).unwrap();
-            let cellbase_hash: H256 = cellbase.hash().unpack();
-
-            assert_eq!(spec_hashes.cellbase, cellbase_hash);
-
-            let mut system_cells = HashMap::new();
-            for (index_minus_one, (cell, (output, data))) in spec_hashes
-                .system_cells
-                .iter()
-                .zip(
-                    cellbase
-                        .outputs()
-                        .into_iter()
-                        .zip(cellbase.outputs_data().into_iter())
-                        .skip(1),
-                )
-                .enumerate()
-            {
-                let data_hash: H256 = packed::CellOutput::calc_data_hash(&data.raw_data()).unpack();
-                let type_hash: Option<H256> = output
-                    .type_()
-                    .to_opt()
-                    .map(|script| script.calc_script_hash().unpack());
-                assert_eq!(index_minus_one + 1, cell.index, "{}", bundled_spec_err);
-                assert_eq!(cell.data_hash, data_hash, "{}", bundled_spec_err);
-                assert_eq!(cell.type_hash, type_hash, "{}", bundled_spec_err);
-                system_cells.insert(cell.index, cell.path.as_str());
-            }
-
-            // dep group tx should be the first tx except cellbase
-            let dep_group_tx = block.transaction(1).unwrap();
-
-            // input index of dep group tx
-            let dep_group_tx_input_index = system_cells.len() + 1;
-            let input_capacity: Capacity = cellbase
-                .output(dep_group_tx_input_index)
-                .unwrap()
-                .capacity()
-                .unpack();
-            let outputs_capacity = dep_group_tx
-                .outputs()
-                .into_iter()
-                .map(|output| Unpack::<Capacity>::unpack(&output.capacity()))
-                .try_fold(Capacity::zero(), Capacity::safe_add)
-                .unwrap();
-            // capacity for input and outpus should be same
-            assert_eq!(input_capacity, outputs_capacity);
-
-            // dep group tx has only one input
-            assert_eq!(dep_group_tx.inputs().len(), 1);
-
-            // all dep groups should be in the spec file
-            assert_eq!(
-                dep_group_tx.outputs_data().len(),
-                spec_hashes.dep_groups.len(),
-                "{}",
-                bundled_spec_err
-            );
-
-            for (i, output_data) in dep_group_tx.outputs_data().into_iter().enumerate() {
-                let dep_group = &spec_hashes.dep_groups[i];
-
-                // check the tx hashes of dep groups in spec file
-                let tx_hash = dep_group.tx_hash.pack();
-                assert_eq!(tx_hash, dep_group_tx.hash(), "{}", bundled_spec_err);
-
-                let out_point_vec =
-                    packed::OutPointVec::from_slice(&output_data.raw_data()).unwrap();
-
-                // all cells included by a dep group should be list in the spec file
-                assert_eq!(
-                    out_point_vec.len(),
-                    dep_group.included_cells.len(),
-                    "{}",
-                    bundled_spec_err
-                );
-
-                for (j, out_point) in out_point_vec.into_iter().enumerate() {
-                    let dep_path = &dep_group.included_cells[j];
-
-                    // dep groups out_point should point to cellbase
-                    assert_eq!(cellbase.hash(), out_point.tx_hash(), "{}", bundled_spec_err);
-
-                    let index_in_cellbase: usize = out_point.index().unpack();
-
-                    // check index for included cells in dep groups
-                    assert_eq!(
-                        system_cells[&index_in_cellbase], dep_path,
-                        "{}",
-                        bundled_spec_err
-                    );
-                }
-            }
-        }
-    }
-
-    #[test]
-    fn test_default_params() {
-        let params: Params = toml::from_str("").unwrap();
-        let expected = Params::default();
-        assert_eq!(params, expected);
-
-        let test_params: &str = r#"
-            genesis_epoch_length = 100
-        "#;
-
-        let params: Params = toml::from_str(&test_params).unwrap();
-        let expected = Params {
-            genesis_epoch_length: Some(100),
-            ..Default::default()
-        };
-
-        assert_eq!(params, expected);
-
-        let test_params: &str = r#"
-            max_block_bytes = 100
-        "#;
-
-        let params: Params = toml::from_str(&test_params).unwrap();
-        let expected = Params {
-            max_block_bytes: Some(100),
-            ..Default::default()
-        };
-
-        assert_eq!(params, expected);
-
-        let test_params: &str = r#"
-            max_block_proposals_limit = 100
-        "#;
-
-        let params: Params = toml::from_str(&test_params).unwrap();
-        let expected = Params {
-            max_block_proposals_limit: Some(100),
-            ..Default::default()
-        };
-
-        assert_eq!(params, expected);
-
-        let test_params: &str = r#"
-            orphan_rate_target = [1, 40]
-        "#;
-
-        let params: Params = toml::from_str(&test_params).unwrap();
-        let expected = Params {
-            orphan_rate_target: Some((1, 40)),
-            ..Default::default()
-        };
-
-        assert_eq!(params, expected);
-    }
-
-    #[test]
-    fn test_params_skip_serializing_if_option_is_none() {
-        let default = Params::default();
-
-        let serialized: Vec<u8> = toml::to_vec(&default).unwrap();
-        let except: Vec<u8> = vec![];
-
-        assert_eq!(serialized, except);
-    }
-
-    #[test]
-    fn test_default_genesis_epoch_ext() {
-        use ckb_types::core::EpochExt;
-        use ckb_types::{packed, U256};
-
-        let params = Params::default();
-        let compact_target = 0x1a08a97e;
-        let genesis_epoch_length = 1743;
-
-        let genesis_epoch_ext = build_genesis_epoch_ext(
-            params.initial_primary_epoch_reward(),
-            compact_target,
-            genesis_epoch_length,
-            params.epoch_duration_target(),
-            params.orphan_rate_target(),
-        );
-
-        // hard code mainnet
-        let expected = EpochExt::new_builder()
-            .number(0)
-            .base_block_reward(Capacity::shannons(110029157726))
-            .remainder_reward(Capacity::shannons(1390))
-            .previous_epoch_hash_rate(U256::from(0x3aa602ee1f497u64))
-            .last_block_hash_in_previous_epoch(packed::Byte32::zero())
-            .start_number(0)
-            .length(genesis_epoch_length)
-            .compact_target(compact_target)
-            .build();
-
-        assert_eq!(genesis_epoch_ext, expected);
-    }
-}
